/*
---

name: Jx.Splitter

description: A Jx.Splitter creates two or more containers within a parent container and provides user control over the size of the containers.

license: MIT-style license.

requires:
 - Jx.Layout

optional:
 - More/Drag

provides: [Jx.Splitter]

css:
 - splitter

...
 */
// $Id$
/**
 * Class: Jx.Splitter
 *
 * Extends: <Jx.Object>
 *
 * a Jx.Splitter creates two or more containers within a parent container
 * and provides user control over the size of the containers.  The split
 * can be made horizontally or vertically.
 *
 * A horizontal split creates containers that divide the space horizontally
 * with vertical bars between the containers.  A vertical split divides
 * the space vertically and creates horizontal bars between the containers.
 *
 * Example:
 * (code)
 * (end)
 * 
 * Locale Keys:
 * - splitter.barToolTip
 *
 * License:
 * Copyright (c) 2008, DM Solutions Group Inc.
 *
 * This file is licensed under an MIT style license
 */
define("jx/splitter", ['../base','./object','./layout','./splitter/snap','require'],
       function(base, jxObject, Layout, Snap, require){
    
    var splitter  = new Class({
        Extends: jxObject,
        Family: 'Jx.Splitter',
        /**
         * Property: domObj
         * {HTMLElement} the element being split
         */
        domObj: null,
        /**
         * Property: elements
         * {Array} an array of elements that are displayed in each of the split
         * areas
         */
        elements: null,
        /**
         * Property: bars
         * {Array} an array of the bars between each of the elements used to
         * resize the split areas.
         */
        bars: null,
        /**
         * Property: firstUpdate
         * {Boolean} track the first resize event so that unexposed Jx things
         * can be forced to calculate their size the first time they are exposed.
         */
        firstUpdate: true,
        options: {
            /* Option: useChildren
             * {Boolean} if set to true, then the children of the
             * element to be split are used as the elements.  The default value is
             * false.  If this is set, then the elements and splitInto options
             * are ignored.
             */
            useChildren: false,
            /* Option: splitInto
             * {Integer} the number of elements to split the domObj into.
             * If not set, then the length of the elements option is used, or 2 if
             * elements is not specified.  If splitInto is specified and elements
             * is specified, then splitInto is used.  If there are more elements than
             * splitInto specifies, then the extras are ignored.  If there are less
             * elements than splitInto specifies, then extras are created.
             */
            splitInto: 2,
            /* Option: elements
             * {Array} an array of elements to put into the split areas.
             * If splitInto is not set, then it is calculated from the length of
             * this array.
             */
            elements: null,
            /* Option: containerOptions
             * {Array} an array of objects that provide options
             *  for the <Jx.Layout> constraints on each element.
             */
            containerOptions: [],
            /* Option: barOptions
             * {Array} an array of object that provide options for the bars,
             * this array should be one less than the number of elements in the
             * splitter.  The barOptions objects can contain a snap property indicating
             * that a default snap object should be created in the bar and the value
             * of 'before' or 'after' indicates which element it snaps open/shut.
             */
            barOptions: [],
            /* Option: layout
             * {String} either 'horizontal' or 'vertical', indicating the
             * direction in which the domObj is to be split.
             */
            layout: 'horizontal',
            /* Option: snaps
             * {Array} an array of objects which can be used to snap
             * elements open or closed.
             */
            snaps: [],
            /* Option: onStart
             * an optional function to call when a bar starts dragging
             */
            onStart: null,
            /* Option: onFinish
             * an optional function to call when a bar finishes dragging
             */
            onFinish: null
        },
    
        parameters: ['domObj','options'],
    
        /**
         * APIMethod: init
         * Create a new instance of Jx.Splitter
         */
        init: function() {
            //in a global build, snap won't load in time
            //load it now... always reference with a full path in here (include the jx/).
            if (base.global && Snap === undefined) {
                Snap = require('jx/splitter/snap');
            }
            this.domObj = document.id(this.options.domObj);
            this.domObj.addClass('jxSplitContainer');
            var jxLayout = this.domObj.retrieve('jxLayout');
            if (jxLayout) {
                jxLayout.addEvent('sizeChange', this.sizeChanged.bind(this));
            }
    
            this.elements = [];
            this.bars = [];
            var i;
            var nSplits = 2;
            if (this.options.useChildren) {
                this.elements = this.domObj.getChildren();
                nSplits = this.elements.length;
            } else {
                nSplits = this.options.elements ?
                                this.options.elements.length :
                                this.options.splitInto;
                for (i=0; i<nSplits; i++) {
                    var el;
                    if (this.options.elements && this.options.elements[i]) {
                        if (this.options.elements[i].domObj) {
                            el = this.options.elements[i].domObj;
                        } else {
                            el = document.id(this.options.elements[i]);
                        }
                        if (!el) {
                            el = this.prepareElement();
                            el.id = this.options.elements[i];
                        }
                    } else {
                        el = this.prepareElement();
                    }
                    this.elements[i] = el;
                    this.domObj.adopt(this.elements[i]);
                }
            }
            this.elements.each(function(el) { el.addClass('jxSplitArea'); });
            for (i=0; i<nSplits; i++) {
                var jxl = this.elements[i].retrieve('jxLayout');
                if (!jxl) {
                    new Layout(this.elements[i], this.options.containerOptions[i]);
                } else {
                    if (this.options.containerOptions[i]) {
                        jxl.resize(Object.merge({},this.options.containerOptions[i],
                            {position:'absolute'}));
                    } else {
                        jxl.resize({position: 'absolute'});
                    }
                }
            }
    
            for (i=1; i<nSplits; i++) {
                var bar;
                if (this.options.prepareBar) {
                    bar = this.options.prepareBar(i-1);
                } else {
                    bar = this.prepareBar();
                }
                bar.store('splitterObj', this);
                bar.store('leftSide',this.elements[i-1]);
                bar.store('rightSide', this.elements[i]);
                this.elements[i-1].store('rightBar', bar);
                this.elements[i].store('leftBar', bar);
                this.domObj.adopt(bar);
                this.bars[i-1] = bar;
            }
    
            //making dragging dependent on mootools Drag class
            if (Drag != undefined) {
                this.establishConstraints();
            }
    
            for (i=0; i<this.options.barOptions.length; i++) {
                if (!this.bars[i]) {
                    continue;
                }
                var opt = this.options.barOptions[i];
                if (opt && opt.snap && (opt.snap == 'before' || opt.snap == 'after')) {
                    var element;
                    if (opt.snap == 'before') {
                        element = this.bars[i].retrieve('leftSide');
                    } else if (opt.snap == 'after') {
                        element = this.bars[i].retrieve('rightSide');
                    }
                    var snap;
                    var snapEvents;
                    if (opt.snapElement) {
                        snap = opt.snapElement;
                        snapEvents = opt.snapEvents || ['click', 'dblclick'];
                    } else {
                        snap = this.bars[i];
                        snapEvents = opt.snapEvents || ['dblclick'];
                    }
                    if (!snap.parentNode) {
                        this.bars[i].adopt(snap);
                    }
                    new Snap(snap, element, this, snapEvents);
                }
            }
    
            for (i=0; i<this.options.snaps.length; i++) {
                if (this.options.snaps[i]) {
                    new Snap(this.options.snaps[i], this.elements[i], this);
                }
            }
    
            this.sizeChanged();
        },
        /**
         * Method: prepareElement
         * Prepare a new, empty element to go into a split area.
         *
         * Returns:
         * {HTMLElement} an HTMLElement that goes into a split area.
         */
        prepareElement: function(){
            var o = new Element('div', {styles:{position:'absolute'}});
            return o;
        },
    
        /**
         * Method: prepareBar
         * Prepare a new, empty bar to go into between split areas.
         *
         * Returns:
         * {HTMLElement} an HTMLElement that becomes a bar.
         */
        prepareBar: function() {
            var o = new Element('div', {
                'class': 'jxSplitBar'+this.options.layout.capitalize(),
                'title': this.getText({set:'Jx',key:'splitter',value:'barToolTip'})
            });
            return o;
        },
    
        /**
         * Method: establishConstraints
         * Setup the initial set of constraints that set the behaviour of the
         * bars between the elements in the split area.
         */
        establishConstraints: function() {
            var modifiers = {x:null,y:null};
            var fn;
            if (this.options.layout == 'horizontal') {
                modifiers.x = "left";
                fn = this.dragHorizontal;
            } else {
                modifiers.y = "top";
                fn = this.dragVertical;
            }
            if (typeof Drag != 'undefined') {
                this.bars.each(function(bar){
                    var mask;
                    new Drag(bar, {
                        //limit: limit,
                        modifiers: modifiers,
                        onSnap : (function(obj) {
                            obj.addClass('jxSplitBarDrag');
                            this.fireEvent('snap',[obj]);
                        }).bind(this),
                        onCancel: (function(obj){
                            mask.destroy();
                            this.fireEvent('cancel',[obj]);
                        }).bind(this),
                        onDrag: (function(obj, event){
                            fn.apply(this,[obj]);
                            this.fireEvent('drag',[obj,event]);
                        }).bind(this),
                        onComplete : (function(obj) {
                            mask.destroy();
                            obj.removeClass('jxSplitBarDrag');
                            if (obj.retrieve('splitterObj') != this) {
                                return;
                            }
                            fn.apply(this,[obj]);
                            this.fireEvent('complete',[obj]);
                            this.fireEvent('finish',[obj]);
                        }).bind(this),
                        onBeforeStart: (function(obj) {
                            this.fireEvent('beforeStart',[obj]);
                            mask = new Element('div',{'class':'jxSplitterMask'}).inject(obj, 'after');
                        }).bind(this),
                        onStart: (function(obj, event) {
                            this.fireEvent('start',[obj, event]);
                        }).bind(this)
                    });
                }, this);
            }
        },
    
        /**
         * Method: dragHorizontal
         * In a horizontally split container, handle a bar being dragged left or
         * right by resizing the elements on either side of the bar.
         *
         * Parameters:
         * obj - {HTMLElement} the bar that was dragged
         */
        dragHorizontal: function(obj) {
            var leftEdge = parseInt(obj.style.left,10);
            var leftSide = obj.retrieve('leftSide');
            var rightSide = obj.retrieve('rightSide');
            var leftJxl = leftSide.retrieve('jxLayout');
            var rightJxl = rightSide.retrieve('jxLayout');
    
            var paddingLeft = this.domObj.measure(function(){
                var m = this.getSizes(['padding'], ['left']);
                return m.padding.left;
            });
    
            /* process right side first */
            var rsLeft, rsWidth, rsRight;
    
            var size = obj.retrieve('size');
            if (!size) {
                size = obj.getBorderBoxSize();
                obj.store('size',size);
            }
            rsLeft = leftEdge + size.width - paddingLeft;
    
            var parentSize = this.domObj.getContentBoxSize();
    
            if (rightJxl.options.width != null) {
                rsWidth = rightJxl.options.width + rightJxl.options.left - rsLeft;
                rsRight = parentSize.width - rsLeft - rsWidth;
            } else {
                rsWidth = parentSize.width - rightJxl.options.right - rsLeft;
                rsRight = rightJxl.options.right;
            }
    
            /* enforce constraints on right side */
            if (rsWidth < 0) {
                rsWidth = 0;
            }
    
            if (rsWidth < rightJxl.options.minWidth) {
                rsWidth = rightJxl.options.minWidth;
            }
            if (rightJxl.options.maxWidth >= 0 && rsWidth > rightJxl.options.maxWidth) {
                rsWidth = rightJxl.options.maxWidth;
            }
    
            rsLeft = parentSize.width - rsRight - rsWidth;
            leftEdge = rsLeft - size.width;
    
            /* process left side */
            var lsLeft, lsWidth;
            lsLeft = leftJxl.options.left;
            lsWidth = leftEdge - lsLeft;
    
            /* enforce constraints on left */
            if (lsWidth < 0) {
                lsWidth = 0;
            }
            if (lsWidth < leftJxl.options.minWidth) {
                lsWidth = leftJxl.options.minWidth;
            }
            if (leftJxl.options.maxWidth >= 0 &&
                lsWidth > leftJxl.options.maxWidth) {
                lsWidth = leftJxl.options.maxWidth;
            }
    
            /* update the leftEdge to accomodate constraints */
            if (lsLeft + lsWidth != leftEdge) {
                /* need to update right side, ignoring constraints because left side
                   constraints take precedence (arbitrary decision)
                 */
                leftEdge = lsLeft + lsWidth;
                var delta = leftEdge + size.width - rsLeft;
                rsLeft += delta;
                rsWidth -= delta;
            }
    
            /* put bar in its final location based on constraints */
            obj.style.left = paddingLeft + leftEdge + 'px';
    
            /* update leftSide positions */
            if (leftJxl.options.width == null) {
                parentSize = this.domObj.getContentBoxSize();
                leftSide.resize({right: parentSize.width - lsLeft-lsWidth});
            } else {
                leftSide.resize({width: lsWidth});
            }
    
            /* update rightSide position */
            if (rightJxl.options.width == null) {
                rightSide.resize({left:rsLeft});
            } else {
                rightSide.resize({left: rsLeft, width: rsWidth});
            }
        },
    
        /**
         * Method: dragVertical
         * In a vertically split container, handle a bar being dragged up or
         * down by resizing the elements on either side of the bar.
         *
         * Parameters:
         * obj - {HTMLElement} the bar that was dragged
         */
        dragVertical: function(obj) {
            /* top edge of the bar */
            var topEdge = parseInt(obj.style.top,10);
    
            /* the containers on either side of the bar */
            var topSide = obj.retrieve('leftSide');
            var bottomSide = obj.retrieve('rightSide');
            var topJxl = topSide.retrieve('jxLayout');
            var bottomJxl = bottomSide.retrieve('jxLayout');
    
            var paddingTop = this.domObj.measure(function(){
                var m = this.getSizes(['padding'], ['top']);
                return m.padding.top;
            });
    
    
            /* measure the bar and parent container for later use */
            var size = obj.retrieve('size');
            if (!size) {
                size = obj.getBorderBoxSize();
                obj.store('size', size);
            }
            var parentSize = this.domObj.getContentBoxSize();
    
            /* process top side first */
            var bsTop, bsHeight, bsBottom;
    
            /* top edge of bottom side is the top edge of bar plus the height of the bar */
            bsTop = topEdge + size.height - paddingTop;
    
            if (bottomJxl.options.height != null) {
                /* bottom side height is fixed */
                bsHeight = bottomJxl.options.height + bottomJxl.options.top - bsTop;
                bsBottom = parentSize.height - bsTop - bsHeight;
            } else {
                /* bottom side height is not fixed. */
                bsHeight = parentSize.height - bottomJxl.options.bottom - bsTop;
                bsBottom = bottomJxl.options.bottom;
            }
    
            /* enforce constraints on bottom side */
            if (bsHeight < 0) {
                bsHeight = 0;
            }
    
            if (bsHeight < bottomJxl.options.minHeight) {
                bsHeight = bottomJxl.options.minHeight;
            }
    
            if (bottomJxl.options.maxHeight >= 0 && bsHeight > bottomJxl.options.maxHeight) {
                bsHeight = bottomJxl.options.maxHeight;
            }
    
            /* recalculate the top of the bottom side in case it changed
               due to a constraint.  The bar may have moved also.
             */
            bsTop = parentSize.height - bsBottom - bsHeight;
            topEdge = bsTop - size.height;
    
            /* process left side */
            var tsTop, tsHeight;
            tsTop = topJxl.options.top;
            tsHeight = topEdge - tsTop;
    
            /* enforce constraints on left */
            if (tsHeight < 0) {
                tsHeight = 0;
            }
            if (tsHeight < topJxl.options.minHeight) {
                tsHeight = topJxl.options.minHeight;
            }
            if (topJxl.options.maxHeight >= 0 &&
                tsHeight > topJxl.options.maxHeight) {
                tsHeight = topJxl.options.maxHeight;
            }
    
            /* update the topEdge to accomodate constraints */
            if (tsTop + tsHeight != topEdge) {
                /* need to update right side, ignoring constraints because left side
                   constraints take precedence (arbitrary decision)
                 */
                topEdge = tsTop + tsHeight;
                var delta = topEdge + size.height - bsTop;
                bsTop += delta;
                bsHeight -= delta;
            }
    
            /* put bar in its final location based on constraints */
            obj.style.top = paddingTop + topEdge + 'px';
    
            /* update topSide positions */
            if (topJxl.options.height == null) {
                topSide.resize({bottom: parentSize.height - tsTop-tsHeight});
            } else {
                topSide.resize({height: tsHeight});
            }
    
            /* update bottomSide position */
            if (bottomJxl.options.height == null) {
                bottomSide.resize({top:bsTop});
            } else {
                bottomSide.resize({top: bsTop, height: bsHeight});
            }
        },
        
        /**
         * APIMethod: resize
         * triggers a resize of the splitter.
         */
        resize: function(){
            this.sizeChanged();
        },
        
        /**
         * Method: sizeChanged
         * handle the size of the container being changed.
         */
        sizeChanged: function() {
            if (this.options.layout == 'horizontal') {
                this.horizontalResize();
            } else {
                this.verticalResize();
            }
        },
    
        /**
         * Method: horizontalResize
         * Resize a horizontally layed-out container
         */
        horizontalResize: function() {
            var availableSpace = this.domObj.getContentBoxSize().width;
            var overallWidth = availableSpace;
            var i,e,jxo;
            for (i=0; i<this.bars.length; i++) {
                var bar = this.bars[i];
                var size = bar.retrieve('size');
                if (!size || size.width == 0) {
                    size = bar.getBorderBoxSize();
                    bar.store('size',size);
                }
                availableSpace -= size.width;
            }
    
            var nVariable = 0, w = 0;
            for (i=0; i<this.elements.length; i++) {
                e = this.elements[i];
                jxo = e.retrieve('jxLayout').options;
                if (jxo.width != null) {
                    availableSpace -= parseInt(jxo.width,10);
                } else {
                    w = 0;
                    if (jxo.right != 0 ||
                        jxo.left != 0) {
                        w = e.getBorderBoxSize().width;
                    }
    
                    availableSpace -= w;
                    nVariable++;
                }
            }
    
            if (nVariable == 0) { /* all fixed */
                /* stick all available space in the last one */
                availableSpace += jxo.width;
                jxo.width = null;
                nVariable = 1;
            }
    
            var amount = parseInt(availableSpace / nVariable,10);
            /* account for rounding errors */
            var remainder = availableSpace % nVariable;
    
            var leftPadding = this.domObj.measure(function(){
                var m = this.getSizes(['padding'], ['left']);
                return m.padding.left;
            });
    
            var currentPosition = 0;
    
            for (i=0; i<this.elements.length; i++) {
                 e = this.elements[i];
                 var jxl = e.retrieve('jxLayout');
                 jxo = jxl.options;
                 if (jxo.width != null) {
                     jxl.resize({left: currentPosition});
                     currentPosition += jxo.width;
                 } else {
                     var a = amount;
                     if (nVariable == 1) {
                         a += remainder;
                     }
                     nVariable--;
    
                     if (jxo.right != 0 || jxo.left != 0) {
                         w = e.getBorderBoxSize().width + a;
                     } else {
                         w = a;
                     }
    
                     if (w < 0) {
                         if (nVariable > 0) {
                             amount = amount + w/nVariable;
                         }
                         w = 0;
                     }
                     if (w < jxo.minWidth) {
                         if (nVariable > 0) {
                             amount = amount + (w - jxo.minWidth)/nVariable;
                         }
                         w = jxo.minWidth;
                     }
                     if (jxo.maxWidth >= 0 && w > jxo.maxWidth) {
                         if (nVariable > 0) {
                             amount = amount + (w - jxo.maxWidth)/nVariable;
                         }
                         w = e.options.maxWidth;
                     }
    
                     var r = overallWidth - currentPosition - w;
                     jxl.resize({left: currentPosition, right: r});
                     currentPosition += w;
                 }
                 var rightBar = e.retrieve('rightBar');
                 if (rightBar) {
                     rightBar.setStyle('left', leftPadding + currentPosition);
                     currentPosition += rightBar.retrieve('size').width;
                 }
             }
        },
    
        /**
         * Method: verticalResize
         * Resize a vertically layed out container.
         */
        verticalResize: function() {
            var availableSpace = this.domObj.getContentBoxSize().height;
            var overallHeight = availableSpace;
            var i,e,jxo;
            for (i=0; i<this.bars.length; i++) {
                var bar = this.bars[i];
                var size = bar.retrieve('size');
                if (!size || size.height == 0) {
                    size = bar.getBorderBoxSize();
                    bar.store('size', size);
                }
                availableSpace -= size.height;
            }
    
            var nVariable = 0, h=0;
            for (i=0; i<this.elements.length; i++) {
                e = this.elements[i];
                jxo = e.retrieve('jxLayout').options;
                if (jxo.height != null) {
                    availableSpace -= parseInt(jxo.height,10);
                } else {
                    if (jxo.bottom != 0 || jxo.top != 0) {
                        h = e.getBorderBoxSize().height;
                    }
    
                    availableSpace -= h;
                    nVariable++;
                }
            }
    
            if (nVariable == 0) { /* all fixed */
                /* stick all available space in the last one */
                availableSpace += jxo.height;
                jxo.height = null;
                nVariable = 1;
            }
    
            var amount = parseInt(availableSpace / nVariable,10);
            /* account for rounding errors */
            var remainder = availableSpace % nVariable;
    
            var paddingTop = this.domObj.measure(function(){
                var m = this.getSizes(['padding'], ['top']);
                return m.padding.top;
            });
    
            var currentPosition = 0;
    
            for (i=0; i<this.elements.length; i++) {
                 e = this.elements[i];
                 var jxl = e.retrieve('jxLayout');
                 jxo = jxl.options;
                 if (jxo.height != null) {
                     jxl.resize({top: currentPosition});
                     currentPosition += jxo.height;
                 } else {
                     var a = amount;
                     if (nVariable == 1) {
                         a += remainder;
                     }
                     nVariable--;
    
                     h = 0;
                     if (jxo.bottom != 0 || jxo.top != 0) {
                         h = e.getBorderBoxSize().height + a;
                     } else {
                         h = a;
                     }
    
                     if (h < 0) {
                         if (nVariable > 0) {
                             amount = amount + h/nVariable;
                         }
                         h = 0;
                     }
                     if (h < jxo.minHeight) {
                         if (nVariable > 0) {
                             amount = amount + (h - jxo.minHeight)/nVariable;
                         }
                         h = jxo.minHeight;
                     }
                     if (jxo.maxHeight >= 0 && h > jxo.maxHeight) {
                         if (nVariable > 0) {
                             amount = amount + (h - jxo.maxHeight)/nVariable;
                         }
                         h = jxo.maxHeight;
                     }
    
                     var r = overallHeight - currentPosition - h;
                     jxl.resize({top: currentPosition, bottom: r});
                     currentPosition += h;
                 }
                 var rightBar = e.retrieve('rightBar');
                 if (rightBar) {
                     rightBar.style.top = paddingTop + currentPosition + 'px';
                     currentPosition += rightBar.retrieve('size').height;
                 }
             }
        },
        
        changeText: function (lang) {
            this.parent();
            this.bars.each(function(bar){
                document.id(bar).set('title', this.getText({set:'Jx',key:'splitter',value:'barToolTip'}));
            },this);	
        }
    });
    
    if (base.global) {
        base.global.Splitter = splitter;
    }
    
    return splitter;
    
});